; V24-Tastatur ueber SID
; (C)2022 werner.roemer@t-online.de
;--------------------------------------------------------------------
; V0 = 1200Bd
;
; SID = RX
; SOD = RTS
;
;
; Fuer die Verwendung einer seriellen Tastatur
; bzw. PS/2 Tastatur mit entsprechendem Wandler
;
; 10.12.2022	Feintuning
; 30.11.2022	Software aus Intel-Hexlader von Moppel-4
;
;
;
;
;--------------------------------------------------------------------
;
; Monitorparameter/Unterprogramme
;
v24buf	equ	0a000h		; V24 Eingangsbuffer *)
v24be		equ	0a7ffh		;

zbuf		equ	2d00h			; Zeilenbuffer
zbufe		equ	2d3Fh			; 64 Zeichen
timeout_flg	equ	2d40h			; Timeoutflag
;
srcbeg	equ 	2fbfh			; Startadresse
srcend	equ 	2fc1h			; Endadresse
dstbeg	equ 	2fc3h			; Anfang Zieladresse/Parameter
dstend	equ 	2fc5h			;
rambeg	equ 	2800h			;
adres		equ	2fbbh			; Adresse HI
adrhi		equ	2fbch			; Adresse LO
;
dely1		equ	000bh			; 1ms warten
onesec	equ	001bh			; 1sec warten

;
warm		equ	1003h			; Monitor Warmstart
string	equ	1021h			; Zeichenkette ausgeben
cts		equ	0052h			; Consolenstatus	Z=0 Taste; Z=1 keine Taste
ci		equ	0043h			; Consoleneingabe	Auf Taste warten (A)=ASCII-Taste
co		equ	0049h			; (C) an Monitor
;
;=====================================================================
;
;
		org	8000h			; Testadresse auf CP/M-Moppel

sid:		jmp	rx			; Empfang
;
;
;
;
rx:		call	init			;

main:		call	sid_st		; Taste ?
		jz	loop			; nein
main_l:	call	sid_ci		;
		mov	c,a			;
		cpi	78h			; wenn (A)="x" Ende
		jz	ende			;
		call	co			;
		jmp	main			;
;
loop		lxi	h,msg1		;
		call	string		; Meldung anzeigen
		jmp	main_l
;
ende:		mvi	a,0c0h		; RTS loeschen
		sim				;
		jmp	warm			; zurueck zum Monitor
;
;-----------------------------------------------------------
;
; init	Vorbereitung fuer Empfang auf der CPU Schnittstelle SID/SOD
;		SID Daten empfangen
;		SOD als RTS		1 = gesperrt/geloescht
;					0 = freigabe

init:		mvi	a,0c0h		; RTS loeschen
		sim				;
		xra	a			;
		sta	timeout_flg		; Timeout loeschen
		lxi	h,msg0		;
		call	string		; Startmeldung
		ret
;
;----------------------------------------------------------
;
; SID_ST 	Status 	Z=1 keine Taste
;				Z=0 (A) Tastencode
;
sid_st:	mvi	a,01			; Timeout setzen
		sta	timeout_flg		;
		call	cpuin			; Taste ?
		ana	a			;
;		jz	sidst_e		; keine Taste
		;
		; Platzhalter fuer Zeilen-Buffer
		;
sidst_e:	ret				;
;
;----------------------------------------------------------
;
; SID CI  	Eingabe	(A) Tastencode
;		wartet bis Taste gedrueckt ist
;
sid_ci:	xra	a			; Timeout loeschen
		sta	timeout_flg		;
		call	cpuin			;
		ret				; (A) = Tastencode
;
;----------------------------------------------------------
;
; CPUIN (A) Daten ueber die CPU-Schnittstelle entgegennehmen
;
; timeout_flg 	= 0 wartet auf ein Zeichen
;				(A) = Zeichencode ASCII
;
;			= 1 testet ob zeichen vorhanden ist
;				Z = 1 kein Zeichen
;				Z = 0 Zeichen vorhanden (A)
;
;
cpuin:	push	h		;
		push	d		;				#
		mvi	d,08h		; Bitzaehler		#
		mvi	e,00h		; Schieberegister		#
		lxi	h,0fffh	; Wert fuer Timeout
		mvi	a,40h		; RTS auf low		# 7
		sim			;				# 4
		;
sidl:		rim			; SID abfragen		# 4   <-----------------------------+
		ani	80h		;				# 7						:
		jz	sbit		; Startbit bearbeiten	# 7/10					:
					;										:
		lda	timeout_flg	;				# 13	 					:
		ana	a		;				# 4						:
		jz	sidl		; Timeout noch nicht	# 7/10 ---> 45 * 0,326s = 14,6s ->+
		;			; eingeschaltet		#						:
timeout:	dcx	h		;				# 6	 					:
		mov	a,h		;				# 4						:
		cpi	00h		;				# 7						:
		jnz	sidl		; noch nicht abgelaufen # 7/10 ---> 72 * 0,326s = 23,4s ->+
		mov	a,l		;				# 4						:
		cpi	00h		; Timeout			# 7						:
		jnz	sidl		; noch nicht abgelaufen # 7/10 ---> 93 * 0,326s = 30,3s ->+
		xra	a		;				# 7
		jmp	cpuend	; Ende			# 10
		;
sbit:		call	delyht	; halbe BitDauer warten #
inlop:	call	delybd	; mitte Datenbit		#
		rim			;				#
		ani	80h		; MSB maskieren		#
		ora	e		; verknuepfen		#
		rrc			;				#
		mov	e,a		; ins Schieberegister	#
		dcr	d		;				#
		jnz	inlop		;				#
		rlc			;				#
		push	psw		;				#
		call	delybd	; mitte Stopbit		#
		;							#
cpuend:	mvi	a,0c0h	; RTS loeschen		# 7
		sim			;				# 4
		pop	psw		;
		pop	d		;				#
		pop	h		;
		ret			;				#
;
;--------------------------------------------------------------------
;
; Verzoegerungszeit fuer Halbe Bitlaenge
;
; fuer 6,144Mhz Quarz
;
		htime	equ	87		;	# 174 fuer 600Bd	0,833ms
						;	#  87 fuer 1200Bd	0,417ms
						;	#  37 fuer 2400Bd	0,208ms
						;	#  18 fuer 4800Bd 0,104ms
; Bit-Zeit warten
delybd	call	delyht		;
		call	delyht		;
		ret				;
; 1/2 Bit-Zeit warten
delyht:	push	psw			; 	# 12
		mvi	a,htime		;	#  7
delop: 	dcr	a			;	#  4
		jnz	delop			;	# 10/7
		pop	psw			;	# 10
		ret				;	# 10
;
;
;--------------------------------------------------------------------------
;
;
msg0:		db	0dh			;
		db	0ah			;
		db	"start"
		db	0dh			;
		db	0ah			;
		db	00h			;

msg1:		db	0dh			;
		db	0ah			;
		db	"warte"
		db	0dh			;
		db	0ah			;
		db	00h			;
;
;
;--------------------------------------------------------------------
end
'oshonsoft_bookmarks_and_breakpoints_info:,1991
